<?php

namespace Drupal\document_ocr\Services;

/**
 * Document OCR Batch API.
 *
 * @ingroup document_ocr
 */
class Batch {

  /**
   * Document processing.
   */
  public static function process($item, &$context) {
    if (empty($context['results']['error'])) {
      $context['results']['error'] = [];
    }
    try {
      if ($task = \Drupal::entityTypeManager()->getStorage('document_ocr_task')->load($item->id)) {
        if ($document = $task->getDocumentOcr()) {
          $settings = $document->getSettings();
          if ($task->getAttempts() < $settings['attempts'] && ($task->isPending() || $task->isInitiated())) {
            if ($destination = \Drupal::service('document_ocr.process')->runPendingTask($task)) {
              $context['results']['documents'][] = $item->id;
              $context['message'] = t('Processing %name.', ['%name' => $destination->label()]);
            }
            else {
              $error_message = t('Unable to process %name at this time. Please wait before you try processing again.', ['%name' => $document->label()]);
              if (!in_array($error_message, $context['results']['error']) && empty($context['results']['error'])) {
                $context['message'] = $error_message;
                $context['results']['error'][] = $error_message;
              }
            }
          }
          else {
            $error_message = t('Attempt limit reached or document marked as processed for %name.', ['%name' => $task->getFile()->getFilename()]);
            if (!in_array($error_message, $context['results']['error']) && empty($context['results']['error'])) {
              $context['message'] = $error_message;
              $context['results']['error'][] = $error_message;
            }
          }
        }
      }
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * One-time files processing.
   */
  public static function processOneTime($item, &$context) {
    try {
      \Drupal::service('document_ocr.process')->runOneTimeTask($item);
      $context['results']['documents'][] = $item->file->id();
      $context['message'] = t('Processing %name.', ['%name' => $item->file->getFilename()]);
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * Process older entities.
   */
  public static function processOlderEntity($entity, &$context) {
    try {
      $context['results']['documents'][] = $entity->id();
      $context['message'] = t('Processing %label.', ['%label' => $entity->label()]);
      \Drupal::service('document_ocr.process')->processEntity($entity);
    }
    catch (\Exception $e) {
      $context['message'] = $e->getMessage();
      $context['results']['error'][] = $e->getMessage();
    }
  }

  /**
   * Complete callback for file process.
   */
  public static function completeProcessCallback($success, $results, $operations) {
    if (!empty($results['error'])) {
      foreach ($results['error'] as $error_message) {
        self::complete($error_message, $error_message, FALSE, $results);
      }
    }
    else {
      self::complete('1 document processed.', '@count documents processed.', $success, $results['documents']);
    }
  }

  /**
   * Helper method for complete callbacks.
   */
  protected static function complete($singular, $plural, $success, $results) {
    if ($success) {
      $message = \Drupal::translation()->formatPlural(count($results), $singular, $plural);
      \Drupal::messenger()->addMessage($message, 'status', FALSE);
      \Drupal::logger('document_ocr')->notice($message);
    }
    else {
      \Drupal::messenger()->addWarning($singular, 'warning', FALSE);
      \Drupal::logger('document_ocr')->warning($singular);
    }
  }

}
